-- creation.sql --

-- ************************************************** --
--                                                    --
--                   PROJET SGBD                      --
--        Gestion d'une agence immobiliere            --
--                    equipe :                        --
--             EL AFRIT Mohamed Amine                 --
--                   KHABOU Amal                      --
--             MAKHLOUF Mohamed Mehdi                 --
--                                                    --
--                                                    --
-- ************************************************** --


-- creation des entites et des associations de la base de donnees

CREATE SEQUENCE seq_quartier START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_immeuble START WITH 1 INCREMENT BY 1 ;
CREATE SEQUENCE seq_categorie START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_etat START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_personne START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_logement START WITH 100 INCREMENT BY 1;
CREATE SEQUENCE seq_bail START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_facture START WITH 5000 INCREMENT BY 1;


-- tout les PRIMARY KEY doivent etre des champs non vide donc non nul

--************--
--  entites   --
--************--


CREATE TABLE ville (code_postal INT NOT NULL PRIMARY KEY,
       	     	    nom_ville CHAR(20) NOT NULL);

-- ** quartier ** --

CREATE TABLE quartier (num_quartier INT NOT NULL PRIMARY KEY,
       	     	    nom_quartier CHAR(20) NOT NULL,
					code_postal INT NOT NULL,
					FOREIGN KEY (code_postal) REFERENCES ville) ;

-- ** immeuble ** --

--modification: etat est par defaut 'dispo' au lieu de NULL
CREATE TABLE immeuble (num_immeuble INT NOT NULL PRIMARY KEY,
       	     	      num_quartier INT NOT NULL,
       	     	      nom_immeuble CHAR(20) DEFAULT 'immeuble',
				      date_cons DATE,
				      date_last_refection DATE,
				      complement_adresse CHAR(40) DEFAULT 'complement adresse',
				      charges INT NOT NULL,
				      etat CHAR(10)  DEFAULT 'dispo',
				      check (date_last_refection >= date_cons),
		 		      FOREIGN KEY (num_quartier) REFERENCES quartier);

-- ** categorie ** --


CREATE TABLE categorie (num_categorie INT NOT NULL PRIMARY KEY,
       	     	       nom_categorie CHAR(10) NOT NULL,
				       surface INT DEFAULT 0,
				       loyer_min INT DEFAULT 0,
				       loyer_moy INT DEFAULT 0,
				       loyer_max INT DEFAULT 0);

-- ** etat ** --

CREATE TABLE etat (num_etat INT NOT NULL PRIMARY KEY,
       	     	  nom_etat CHAR(10) NOT NULL);


-- ** personne ** --

--- proposition:
-- on commence par un nss quelconque et on incremente par une valeur quelconque 
-- car en realite les nss ne sont pas successive dans une base de donnee
-- il faut voir si cela ne pause pas trop de difficulte pour remplir 'definit_locataire'

CREATE TABLE personne (nss INT NOT NULL PRIMARY KEY,
       	     	      nom CHAR(15) NOT NULL,
				      prenom CHAR(15) NOT NULL,
				      date_naissance DATE,
				      profession CHAR(30) DEFAULT 'NC',
				      revenu INT DEFAULT 0,
				      situation_prof CHAR(20) DEFAULT 'NC',
				      prestations_sociale INT DEFAULT 0);

-- ** logement ** -- 

--modification: disponibilite est par defaut 'dispo' au lieu de NULL
CREATE TABLE logement (num_logement INT NOT NULL PRIMARY KEY,
       	     	    num_immeuble INT NOT NULL,
				    num_categorie INT NOT NULL,
				    num_etat INT NOT NULL,
				    nss INT NOT NULL,
				    adresse_prop CHAR(80) DEFAULT 'NC',
				    date_last_refection DATE,
				    disponibilite CHAR(10) DEFAULT 'dispo',
				    FOREIGN KEY  (num_immeuble) references immeuble,
				    FOREIGN KEY (num_categorie) references categorie,
				    FOREIGN KEY (num_etat) references etat,
				    FOREIGN KEY (nss) references personne);

-- ** bail ** --

CREATE TABLE bail (num_bail INT NOT NULL PRIMARY KEY,
       	     	  num_logement INT NOT NULL,
       	     	  date_signature DATE,
				  caution INT DEFAULT 0,
				  date_fin DATE NOT NULL,
				  FOREIGN KEY (num_logement) REFERENCES logement);

-- plus de duree plutot date_fin

-- ** facture ** --

CREATE TABLE facture (num_facture INT NOT NULL PRIMARY KEY,
       	     	    num_bail INT NOT NULL,
       	     	    date_payement DATE DEFAULT NULL,
				    charges_mens INT DEFAULT 0,
				    loyer INT NOT NULL,
				    FOREIGN KEY (num_bail) REFERENCES bail);

-- ajout du champ "loyer"

--****************--
--  association   --
--****************--

-- ** signe_par ** --
 
CREATE TABLE signe_par (nss INT NOT NULL,
			    num_bail INT NOT NULL,
 		        type_sign CHAR(25) DEFAULT 'NC',
		        adresse_sign CHAR(60) DEFAULT 'AS',
				PRIMARY KEY (nss, num_bail),
				FOREIGN KEY (nss) REFERENCES personne,
				FOREIGN KEY (num_bail) REFERENCES bail);

-- ** definit_locataire ** --

CREATE TABLE definit_locataire (nss INT NOT NULL,
       	     	    num_bail INT NOT NULL,
			        PRIMARY KEY (nss, num_bail),
		            FOREIGN KEY (nss) REFERENCES personne,
		       	    FOREIGN KEY (num_bail) REFERENCES bail);

-- ** est_a_charge ** --

CREATE TABLE est_a_charge (nss1 INT NOT NULL,
       	     		nss2 INT NOT NULL,
			        PRIMARY KEY (nss1),
		            FOREIGN KEY (nss1) REFERENCES personne);



--****************--
--   VUES         --
--****************--

--Les attributs etat pour immeuble et disponibilite pour logement sont inutiles
--pour l'utilisateur, ça permet d'effectuer une suppression logique.
--la supression physique n'est pas possible car on aura besoin des historiques.

CREATE VIEW immeuble_reel(num_immeuble,num_quartier,nom_immeuble,
			date_cons,date_last_refection,complement_adresse,charges)
AS (SELECT num_immeuble, num_quartier, nom_immeuble, date_cons, 
		date_last_refection, complement_adresse, charges 
   	   		FROM immeuble  
	   		WHERE (etat = 'dispo'));


CREATE VIEW logement_reel(num_logement,num_immeuble,num_categorie,
			num_etat,nss,adresse_prop,date_last_refection) 
AS (SELECT num_logement, num_immeuble, num_categorie, num_etat, nss,
		adresse_prop, date_last_refection 
   	   		   FROM logement 
	   		   WHERE (disponibilite = 'dispo'));

--la mise à jour influe sur les vues sans que l'utilisateur connaisse l'origine 
--des modifications 
